<?php

/**
 * This file contains QUI\ERP\Employee\Search
 */

namespace QUI\ERP\Employee;

use IntlDateFormatter;
use PDO;
use QUI;
use QUI\Utils\Singleton;

use function array_flip;
use function array_map;
use function count;
use function explode;
use function implode;
use function is_string;
use function sort;
use function str_replace;
use function strtotime;
use function trim;

/**
 * Class Search
 *
 * @package QUI\ERP\Employee
 */
class Search extends Singleton
{
    /**
     * @var string
     */
    protected string $order = 'id DESC';

    /**
     * @var array
     */
    protected array $filter = [];

    /**
     * search value
     *
     * @var null|string
     */
    protected ?string $search = null;

    /**
     * search flag for: search only in employee
     *
     * @var bool
     */
    protected bool $onlyEmployee = true;

    /**
     * @var array
     */
    protected array $limit = [0, 20];

    /**
     * Return the db table
     *
     * @return string
     */
    public function table(): string
    {
        return QUI::getDBTableName('users');
    }

    /**
     * Return the db table for the addresses
     *
     * @return string
     */
    public function tableAddress(): string
    {
        return QUI::getDBTableName('users_address');
    }

    /**
     * @return array
     */
    public function getAllowedFields(): array
    {
        return [
            'id',
            'email',
            'username',
            'usergroup',
            'usertitle',
            'firstname',
            'lastname',
            'lang',
            'company',
            'birthday',
            'active',
            'deleted',
            'su',

            'regdate',
            'lastvisit',
            'lastedit',
            'expire'
        ];
    }

    /**
     * Execute the search and return the invoice list
     *
     * @return array
     *
     * @throws QUI\Exception
     */
    public function search(): array
    {
        return $this->executeQueryParams($this->getQuery());
    }

    /**
     * @throws QUI\Exception
     */
    public function searchForGrid(): array
    {
        // select display invoices
        $users = $this->executeQueryParams($this->getQuery());

        // count
        $count = $this->executeQueryParams($this->getQueryCount());
        $count = (int)$count[0]['count'];

        // result
        $result = $this->parseListForGrid($users);
        $Grid = new QUI\Utils\Grid();
        $Grid->setAttribute('page', ($this->limit[0] / $this->limit[1]) + 1);

        return $Grid->parseResult($result, $count);
    }

    /**
     * @param $data
     * @return array
     */
    protected function parseListForGrid($data): array
    {
        $localeCode = QUI::getLocale()->getLocalesByLang(
            QUI::getLocale()->getCurrent()
        );

        $DateFormatter = new IntlDateFormatter(
            $localeCode[0],
            IntlDateFormatter::SHORT,
            IntlDateFormatter::NONE
        );

        $DateFormatterLong = new IntlDateFormatter(
            $localeCode[0],
            IntlDateFormatter::SHORT,
            IntlDateFormatter::SHORT
        );

        $result = [];
        $Groups = QUI::getGroups();
        $Users = QUI::getUsers();

        foreach ($data as $entry) {
            $entry['usergroup'] = trim($entry['usergroup'], ',');
            $entry['usergroup'] = explode(',', $entry['usergroup']);
            $entry['usergroup'] = array_map(function ($groupId) {
                return $groupId;
            }, $entry['usergroup']);

            $groups = array_map(function ($groupId) use ($Groups) {
                try {
                    $Group = $Groups->get($groupId);

                    return $Group->getName();
                } catch (QUI\Exception) {
                }

                return '';
            }, $entry['usergroup']);

            sort($groups);
            $groups = implode(', ', $groups);
            $groups = str_replace(',,', '', $groups);
            $groups = trim($groups, ',');

            if (empty($entry['firstname']) || empty($entry['lastname'])) {
                try {
                    $User = $Users->get($entry['id']);
                    $Address = $User->getStandardAddress();

                    if ($Address->getAttribute('firstname')) {
                        $entry['firstname'] = $Address->getAttribute('firstname');
                    }

                    if ($Address->getAttribute('lastname')) {
                        $entry['lastname'] = $Address->getAttribute('lastname');
                    }
                } catch (QUI\Exception) {
                }
            }

            $result[] = [
                'id' => (int)$entry['id'],
                'uuid' => (int)$entry['uuid'],
                'status' => !!$entry['active'],
                'username' => $entry['username'],
                'firstname' => $entry['firstname'],
                'lastname' => $entry['lastname'],
                'email' => $entry['email'],
                'regdate' => $DateFormatterLong->format($entry['regdate']),

                'usergroup_display' => $groups,
                'usergroup' => $entry['usergroup']
            ];
        }

        return $result;
    }

    //region query stuff

    /**
     * @return array
     */
    protected function getQueryCount(): array
    {
        return $this->getQuery(true);
    }

    /**
     * @param bool $count - Use count select, or not
     * @return array
     */
    protected function getQuery(bool $count = false): array
    {
        $table = $this->table();
        $order = $this->order;

        // limit
        $limit = '';

        if ($this->limit && isset($this->limit[0]) && isset($this->limit[1])) {
            $start = $this->limit[0];
            $end = $this->limit[1];
            $limit = " LIMIT $start,$end";
        }


        // filter checks
        $filterList = [
            'userId',
            'username',
            'firstname',
            'lastname',
            'email',
            'usergroup'
        ];

        $searchFilters = [];

        $filterReset = function () use ($filterList) {
            foreach ($filterList as $filter) {
                if (isset($this->filter[$filter]) && $this->filter[$filter]) {
                    return false;
                }
            }

            return true;
        };

        if ($filterReset()) {
            foreach ($filterList as $filter) {
                if (isset($this->filter[$filter])) {
                    unset($this->filter[$filter]);
                }
            }

            $searchFilters = $filterList;
        } else {
            foreach ($filterList as $filter) {
                if (isset($this->filter[$filter]) && $this->filter[$filter]) {
                    $searchFilters[] = $filter;
                }
            }
        }


        // create query
        if (empty($this->filter) && empty($this->search)) {
            $where = '';

            if ($this->onlyEmployee) {
                try {
                    $employeeGroup = Employees::getInstance()->getEmployeeGroupId();
                    $where = " WHERE usergroup LIKE '%,$employeeGroup,%'";
                } catch (QUI\Exception $Exception) {
                    QUI\System\Log::addError($Exception->getMessage());
                }
            }

            if ($count) {
                return [
                    'query' => " SELECT COUNT(id) AS count FROM $table $where",
                    'binds' => []
                ];
            }

            return [
                'query' => "
                    SELECT *
                    FROM {$table}
                    {$where}
                    ORDER BY {$order}
                    {$limit}
                ",
                'binds' => []
            ];
        }

        $where = [];
        $binds = [];

        // filter
        $fc = 0;

        if ($this->onlyEmployee) {
            try {
                $employeeGroup = Employees::getInstance()->getEmployeeGroupId();
                $where[] = 'usergroup LIKE :employeeGroups';

                $binds['employeeGroups'] = [
                    'value' => '%,' . $employeeGroup . ',%',
                    'type' => PDO::PARAM_STR
                ];
            } catch (QUI\Exception $Exception) {
                QUI\System\Log::addError($Exception->getMessage());
            }
        }


        // create filter mysql
        foreach ($this->filter as $filter => $value) {
            if (empty($value)) {
                continue;
            }

            $bind = 'filter' . $fc;

            if ($filter === 'regdate_from') {
                $where[] = 'regdate >= :' . $bind;

                $binds[$bind] = [
                    'value' => (int)strtotime($value),
                    'type' => PDO::PARAM_INT
                ];
                continue;
            }

            if ($filter === 'regdate_to') {
                $where[] = 'regdate <= :' . $bind;

                $binds[$bind] = [
                    'value' => (int)strtotime($value),
                    'type' => PDO::PARAM_INT
                ];
                continue;
            }

            switch ($filter) {
                case 'lastvisit_from':
                    $where[] = 'lastvisit >= :' . $bind;
                    break;

                case 'lastvisit_to':
                    $where[] = 'lastvisit <= :' . $bind;
                    break;

                case 'lastedit_from':
                    $where[] = 'lastedit >= :' . $bind;
                    break;

                case 'lastedit_to':
                    $where[] = 'lastedit <= :' . $bind;
                    break;

                case 'expire_from':
                    $where[] = 'expire >= :' . $bind;
                    break;

                case 'expire_to':
                    $where[] = 'expire <= :' . $bind;
                    break;

                default:
                    continue 2;
            }

            $binds[$bind] = [
                'value' => $value,
                'type' => PDO::PARAM_STR
            ];

            $fc++;
        }


        if (!empty($this->search)) {
            $searchWhere = [];

            foreach ($searchFilters as $filter) {
                if ($filter === 'userId') {
                    $filter = 'id';
                }

                if ($filter === 'group') {
                    $filter = 'usergroup';
                }

                $searchWhere[] = $filter . ' LIKE :search';
            }

            $where[] = '(' . implode(' OR ', $searchWhere) . ')';

            $binds['search'] = [
                'value' => '%' . $this->search . '%',
                'type' => PDO::PARAM_STR
            ];
        }

        $whereQuery = 'WHERE ' . implode(' AND ', $where);

        if (!count($where)) {
            $whereQuery = '';
        }

        if ($count) {
            return [
                "query" => "
                    SELECT COUNT(id) AS count
                    FROM {$table}
                    {$whereQuery}
                ",
                'binds' => $binds
            ];
        }

        return [
            "query" => "
                SELECT *
                FROM {$table}
                {$whereQuery}
                ORDER BY {$order}
                {$limit}
            ",
            'binds' => $binds
        ];
    }

    /**
     * @param array $queryData
     * @return array
     * @throws QUI\Exception
     */
    protected function executeQueryParams(array $queryData = []): array
    {
        $PDO = QUI::getDataBase()->getPDO();
        $binds = $queryData['binds'];
        $query = $queryData['query'];

        $Statement = $PDO->prepare($query);

        foreach ($binds as $var => $bind) {
            $Statement->bindValue($var, $bind['value'], $bind['type']);
        }

        try {
            $Statement->execute();

            return $Statement->fetchAll(PDO::FETCH_ASSOC);
        } catch (\Exception $Exception) {
            QUI\System\Log::writeRecursive($Exception);
            QUI\System\Log::writeRecursive($query);
            QUI\System\Log::writeRecursive($binds);
            throw new QUI\Exception('Something went wrong');
        }
    }

    /**
     * Set the order
     *
     * @param string $order
     */
    public function order(string $order): void
    {
        $allowed = [];

        foreach ($this->getAllowedFields() as $field) {
            $allowed[] = $field;
            $allowed[] = $field . ' ASC';
            $allowed[] = $field . ' asc';
            $allowed[] = $field . ' DESC';
            $allowed[] = $field . ' desc';
        }

        $order = trim($order);
        $allowed = array_flip($allowed);

        if (isset($allowed[$order])) {
            $this->order = $order;
        }
    }

    /**
     * Set the limit
     *
     * @param integer|string $from - start
     * @param integer|string $count - count of entries
     */
    public function limit(int|string $from, int|string $count): void
    {
        $this->limit = [(int)$from, (int)$count];
    }

    //endregion

    //region filter

    /**
     * Set a filter
     *
     * @param string $filter
     * @param array|string $value
     */
    public function setFilter(string $filter, array|string $value): void
    {
        if ($filter === 'search' && is_string($value)) {
            $this->search = $value;

            return;
        }

        if ($filter === 'userId') {
            $this->filter['userId'] = $value;

            return;
        }

        if ($filter === 'group') {
            $this->filter['usergroup'] = $value;

            return;
        }

        if ($filter === 'regdate_from' || $filter === 'regdate_to') {
            $this->filter[$filter] = $value;

            return;
        }

        $keys = array_flip($this->getAllowedFields());

        if (isset($keys[$filter])) {
            $this->filter[$filter] = $value;
        }
    }

    /**
     * Clear all filters
     */
    public function clearFilter(): void
    {
        $this->filter = [];
    }

    /**
     * set the flag for searching only in the employee group
     */
    public function searchOnlyInEmployee(): void
    {
        $this->onlyEmployee = true;
    }

    /**
     * set the flag for searching in all groups
     */
    public function searchInAllGroups(): void
    {
        $this->onlyEmployee = false;
    }

    //endregion
}
